'use strict';

var log = require('./../core/extensions/log'),
    NodeHTTPServer = function() {};

require('util').inherits(NodeHTTPServer, require('./../core/ServerInterface'));

NodeHTTPServer.prototype.constructor = function(config) {
    this._server = this._create_server(config, config.listener);
    this._server_listen_parameters = this._build_listen_parameters(config);

    delete config.listener;
    this._config = config;
};
NodeHTTPServer.prototype.server = function() {
    return this._server;
};
NodeHTTPServer.prototype.configure = function(configurator) {
    configurator(this._server);

    return this;
};

NodeHTTPServer.prototype.start = function(callback) {
    var params = this._server_listen_parameters.slice(0),
        config = this._config;
    
    if(typeof callback === 'function') {
        params.push(function() {
            callback(config);
        });
    }
    
    this._server.listen.apply(this._server, params);
    
    return this;
};

NodeHTTPServer.prototype.stop = function(callback) {
    this._server.close(callback);

    return this;
};

NodeHTTPServer.prototype._create_server = function(config, listener) {
    var read = function(path, options) {
            return require('fs').readFileSync(path, options);
        },
        credentials = config.ssl;

    if(!require('lodash/isPlainObject')(credentials)) {
        return require('http').createServer(listener);
    }

    if(credentials.pfx) {
        credentials = {
            pfx: read(credentials.pfx, 'utf8')
        };
    } else if(credentials.key && credentials.cert) {
        credentials = {
            key: read(credentials.key, 'utf8'),
            cert: read(credentials.cert, 'utf8')
        };
    } else {
        log.error('extension:ifnode-http', 'Wrong https credentials');
    }

    return require('https').createServer(credentials, listener);
};

NodeHTTPServer.prototype._build_listen_parameters = function(config) {
    var host = config.host,
        listen_params = [];

    if(config.port) {
        listen_params.push(config.port);
    }

    if(host !== '127.0.0.1' || host !== 'localhost') {
        listen_params.push(config.host);
    }

    return listen_params;
};

module.exports = NodeHTTPServer;
